"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const task_scheduler_1 = require("@microsoft/task-scheduler");
const npmTask_1 = require("./npmTask");
const backfill_1 = require("../cache/backfill");
const filterPackages_1 = require("./filterPackages");
const logger_1 = require("../logger");
const cacheTasks_1 = require("../cache/cacheTasks");
async function runTasks(options) {
    const { graph, workspace, context, config } = options;
    let pipeline = task_scheduler_1.createPipeline(graph, {
        // dummy logger for task-scheduler because lage already has logger for its tasks
        logger: {
            error: (_msg) => { },
            log: (_msg) => { },
        },
        exit: (code) => { },
    });
    const taskNames = Object.keys(config.pipeline);
    // After all the npm tasks are added, add the cache put task
    pipeline = pipeline.addTask({
        name: cacheTasks_1.CacheHashTask,
        run: async (_location, _stdout, _stderr, pkg) => {
            await backfill_1.cacheHash(workspace.allPackages[pkg], config);
            return true;
        },
    });
    pipeline = pipeline.addTask({
        name: cacheTasks_1.CacheFetchTask,
        deps: [cacheTasks_1.CacheHashTask],
        run: async (_location, _stdout, _stderr, pkg) => {
            await backfill_1.cacheFetch(workspace.allPackages[pkg], config);
            return true;
        },
    });
    for (const [task, taskDeps] of Object.entries(config.pipeline)) {
        const deps = taskDeps.filter((dep) => !dep.startsWith("^"));
        const topoDeps = taskDeps
            .filter((dep) => dep.startsWith("^"))
            .map((dep) => dep.slice(1));
        pipeline = pipeline.addTask({
            name: task,
            deps: [...(deps ? deps : []), cacheTasks_1.CacheFetchTask],
            topoDeps,
            run: async (_location, _stdout, _stderr, pkg) => {
                await npmTask_1.npmTask(task, workspace.allPackages[pkg], config, context);
                return true;
            },
        });
    }
    // After all the npm tasks are added, add the cache put task
    pipeline = pipeline.addTask({
        name: cacheTasks_1.CachePutTask,
        deps: config.command,
        run: async (_location, _stdout, _stderr, pkg) => {
            const failedStats = context.measures.taskStats.some((s) => s.pkg === pkg && s.status === "failed");
            if (!failedStats) {
                await backfill_1.cachePut(workspace.allPackages[pkg], config);
            }
            return true;
        },
    });
    // Filter packages per --scope and command(s)
    const filteredPackages = filterPackages_1.filterPackages({
        root: workspace.root,
        allPackages: workspace.allPackages,
        deps: config.deps,
        scope: config.scope,
        since: config.since,
        ignore: config.ignore,
    });
    // Set up the longest names of tasks and scripts for nice logging
    logger_1.setTaskLogMaxLengths(Object.keys(workspace.allPackages).reduce((l, pkg) => (l < pkg.length ? pkg.length : l), 0), taskNames.reduce((l, task) => (l < task.length ? task.length : l), 0));
    await pipeline.go({
        packages: filteredPackages,
        tasks: config.cache ? [...config.command, cacheTasks_1.CachePutTask] : config.command,
    });
}
exports.runTasks = runTasks;
